#!/usr/bin/python2.2

## soundcardBackend.py - Contains the backend code needed for redhat-config-soundcard
## Copyright (C) 2002 Red Hat, Inc.
## Copyright (C) 2002 Brent Fox <bfox@redhat.com>

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


import string
import kudzu
import os

class soundcardBackend:
    def __init__(self):
        self.defaultModule = None
        self.newFile = []
        self.readFile()
        self.cardList = []
        self.dspList = []

    def probeCards(self):
        self.cardList = []
        list = kudzu.probe(kudzu.CLASS_AUDIO, kudzu.BUS_PCI, kudzu.PROBE_ALL)

        for card in list:
            #Let's go through the list of cards.  If the driver is 'unknown',
            #don't put the card in the list.
            if card.driver != "unknown":
                self.cardList.append(card)

        return self.cardList

    def getData(self, card):
        device, module, description = card
        maker, model = string.split(description, "|")
        return device, module, maker, model

    def insmod(self, module):
        pass

    def rmmod(self, module):
        pass

    def lsmod(self):
        pass

    def playTestSound(self, module):
        if self.dspList.index(module) == 0:
            device = "/dev/dsp"
        else:
            device = "/dev/dsp%d" % self.dspList.index(module)

        path = "/usr/bin/play"
        pid = os.fork()
        args = [path,  "-d", device, "/usr/share/redhat-config-soundcard/redhat-sample.wav", "-v", "2"]

        if (not pid):
            os.execv(path, args)
            
    def readFile(self):
        fd = open('/etc/modules.conf', 'r')
        lines = fd.readlines()
        fd.close()

        self.newFile = []

        for line in lines:
            #strip out all lines dealing with sound
            if string.find(line, "sound-slot") < 0 or string.strip(line)[0] == "#":
                self.newFile.append(line)

            else:
                if string.find(line, "alias") > -1 and string.find(line, "sound-slot-0") > -1:
                    try:
                        a, b, self.defaultModule = string.split(line)
                    except:
                        self.defaultModule = None

    def getDefaultCard(self):
        for card in self.cardList:
            device, module, maker, model = self.getData(card)
            
            if module == self.defaultModule:
                return card

    def writeFile(self, selectedCard):
        self.selectedCard = selectedCard
        aliasList = []
        aumixList = []
        self.dspList = []
        
        device, self.defaultModule, maker, model = self.getData(self.cardList[selectedCard])
        aliasList.append("alias sound-slot-0 %s\n" % self.defaultModule)
        aumixList.append("post-install sound-slot-0 /bin/aumix-minimal -f /etc/.aumixrc -L >/dev/null 2>&1 || :\n")
        aumixList.append("pre-remove sound-slot-0 /bin/aumix-minimal -f /etc/.aumixrc -S >/dev/null 2>&1 || :\n")
        self.dspList.append(self.defaultModule)

        count = 1
        for card in self.cardList:
            device, module, maker, model = self.getData(card)

            if module == self.defaultModule:
                #We've already written this line, so skip
                pass
            else:
                aliasList.append("alias sound-slot-%d %s\n" % (count, module))
                aumixList.append("post-install sound-slot-%d /bin/aumix-minimal -f /etc/.aumixrc -L >/dev/null 2>&1 || :\n" % count)
                aumixList.append("pre-remove sound-slot-%d /bin/aumix-minimal -f /etc/.aumixrc -S >/dev/null 2>&1 || :\n" % count)
                self.dspList.append(module)
                count = count + 1

        #Let's re-read the file again to make sure we're starting from a clean base
        self.readFile()
        self.newFile = self.newFile + aliasList + aumixList

        fd = open('/etc/modules.conf', 'w')

        for line in self.newFile:
            fd.write(line)

        fd.close()

    def unloadModules(self):
        for card in self.cardList:
            device, module, maker, model = self.getData(card)

            retval = os.system("/sbin/modprobe -r %s" % module)

    def loadModules(self):
        retval = os.system("/sbin/modprobe %s" % self.defaultModule)
        if retval == 0:
            #Modprobe worked
            pass
        else:
            #Modprobe failed
            return self.defaultModule

        for card in self.cardList:
            device, module, maker, model = self.getData(card)

            if module == self.defaultModule:
                #We've already inserted this one, so don't try again
                pass
            else:
                retval = os.system("/sbin/modprobe %s" % module)

                if retval == 0:
                    #Modprobe worked
                    pass
                else:
                    #Modprobe failed
                    return self.defaultModule

        return 0

